@@ -9,6 +9,8 @@ from sanic_jwt import exceptions
 
 from gino_admin import config
 from gino_admin.utils import logger
+from gino_admin.history import log_history_event
+from passlib.hash import pbkdf2_sha256
 
 cfg = config.cfg
 
@@ -20,18 +22,20 @@ def token_validation():
             if not os.getenv("ADMIN_AUTH_DISABLE") == "1":
                 if (
                     not request.cookies
-                    or request.cookies["auth-token"] not in cfg.sessions
+                    or not request.cookies.get("auth-token")
+                    or request.cookies.get("auth-token") not in cfg.sessions
                     or (
                         request.cookies["auth-token"] in cfg.sessions
-                        and cfg.sessions[request.cookies["auth-token"]]
+                        and cfg.sessions[request.cookies["auth-token"]]["user_agent"]
                         != request.headers["User-Agent"]
                     )
                 ):
-                    return r.redirect("/admin/login")
+                    return r.redirect(f"{cfg.route}/login")
                 else:
                     request["session"] = {"_auth": True}
                     return await route(request, *args, **kwargs)
             else:
+                request["session"] = {"_auth": True}
                 return await route(request, *args, **kwargs)
 
         return validate
@@ -39,14 +43,37 @@ def token_validation():
     return decorator
 
 
-def validate_login(request, config):
+async def validate_login(request, _config):
     if request.method == "POST":
         username = str(request.form.get("username"))
+        if not username:
+            message = 'failed attempt to auth. no username provided'
+            request["flash_messages"].append(
+                (
+                    message,
+                    "error",
+                )
+            )
+            log_history_event(request, message, 'system: login')
+            return False
         password = str(request.form.get("password"))
-        admin_user = str(config["ADMIN_USER"])
-        admin_password = str(config["ADMIN_PASSWORD"])
-        if username == admin_user and password == admin_password:
-            return True
+        base_user = cfg.app.config.get('ADMIN_USER')
+        if username == base_user:
+            if password == str(cfg.app.config.get('ADMIN_PASSWORD')):
+                return username
+        user_in_base = await cfg.users_model.get(username)
+        if not user_in_base:
+            request["flash_messages"].append(
+                (
+                    f"No User with login {username}",
+                    "error",
+                )
+            )
+            log_history_event(request, f'failed attempt to auth. wrong user: {username}', 'system: login')
+            return False
+        if pbkdf2_sha256.verify(password, user_in_base.password_hash):
+            return username
+        log_history_event(request, f'failed attempt to auth. wrong password for user: {username}', 'system: login')
     return False
 
 
